@@ -16,7 +16,7 @@
 #include <set>
 
 #include <zypp/base/Iterator.h>
-#include <zypp/APIConfig.h>
+#include <zypp-core/Globals.h>
 
 #include <zypp/Url.h>
 #include <zypp/Locale.h>
@@ -68,13 +68,18 @@ namespace zypp
    * \note Name, baseUrls and mirrorUrl are subject to repo variable replacement
    * (\see \ref RepoVariablesStringReplacer).
    */
-  class RepoInfo : public repo::RepoInfoBase
+  class ZYPP_API RepoInfo : public repo::RepoInfoBase
   {
     friend std::ostream & operator<<( std::ostream & str, const RepoInfo & obj );
 
     public:
       RepoInfo();
-      virtual ~RepoInfo();
+      ~RepoInfo() override;
+
+      RepoInfo(const RepoInfo &) = default;
+      RepoInfo(RepoInfo &&) = default;
+      RepoInfo &operator=(const RepoInfo &) = default;
+      RepoInfo &operator=(RepoInfo &&) = default;
 
       /** Represents no Repository (one with an empty alias). */
       static const RepoInfo noRepo;
@@ -100,9 +105,9 @@ namespace zypp
        */
       void setPriority( unsigned newval_r );
 
-      typedef std::list<Url>          url_set;
-      typedef url_set::size_type      urls_size_type;
-      typedef transform_iterator<repo::RepoVariablesUrlReplacer, url_set::const_iterator> urls_const_iterator;
+      using url_set = std::list<Url>;
+      using urls_size_type = url_set::size_type;
+      using urls_const_iterator = transform_iterator<repo::RepoVariablesUrlReplacer, url_set::const_iterator>;
       /**
        * whether repository urls are available
        */
@@ -154,11 +159,11 @@ namespace zypp
        * To recreate the base URLs list, use \ref setBaseUrl(const Url &) followed
        * by addBaseUrl().
        */
-      void addBaseUrl( const Url &url );
+      void addBaseUrl( Url url );
       /**
        * Clears current base URL list and adds \a url.
        */
-      void setBaseUrl( const Url &url );
+      void setBaseUrl( Url url );
       /**
        * Clears current base URL list and adds an \ref url_set.
        */
@@ -266,7 +271,7 @@ namespace zypp
       void setMetadataPath( const Pathname &path );
 
       /** Whether \ref metadataPath uses \c %AUTO% setup. */
-      bool usesAutoMethadataPaths() const;
+      bool usesAutoMetadataPaths() const;
 
       /**
        * \short Path where this repo packages are cached
@@ -532,13 +537,13 @@ namespace zypp
        * Write a human-readable representation of this RepoInfo object
        * into the \a str stream. Useful for logging.
        */
-      virtual std::ostream & dumpOn( std::ostream & str ) const;
+      std::ostream & dumpOn( std::ostream & str ) const override;
 
       /**
        * Write this RepoInfo object into \a str in a <tr>.repo</tt> file format.
        * Raw values, no variable replacement.
        */
-      virtual std::ostream & dumpAsIniOn( std::ostream & str ) const;
+      std::ostream & dumpAsIniOn( std::ostream & str ) const override;
 
       /**
        * Write an XML representation of this RepoInfo object.
@@ -548,13 +553,16 @@ namespace zypp
        * \param content this argument is ignored (used in other classed derived
        *                from RepoInfoBase.
        */
-      virtual std::ostream & dumpAsXmlOn( std::ostream & str, const std::string & content = "" ) const;
+      std::ostream & dumpAsXmlOn( std::ostream & str, const std::string & content = "" ) const override;
+
+      /** Raw values for RepoManager
+       *  \internal
+       */
+      void getRawGpgChecks( TriBool & g_r, TriBool & r_r, TriBool & p_r ) const;
 
       struct Impl;
     private:
       friend class RepoManager;
-      /** Raw values for RepoManager */
-      void getRawGpgChecks( TriBool & g_r, TriBool & r_r, TriBool & p_r ) const;
 
       /** Pointer to implementation */
       RWCOW_pointer<Impl> _pimpl;
@@ -562,17 +570,17 @@ namespace zypp
   ///////////////////////////////////////////////////////////////////
 
   /** \relates RepoInfo */
-  typedef shared_ptr<RepoInfo> RepoInfo_Ptr;
+  using RepoInfo_Ptr = shared_ptr<RepoInfo>;
   /** \relates RepoInfo */
-  typedef shared_ptr<const RepoInfo> RepoInfo_constPtr;
+  using RepoInfo_constPtr = shared_ptr<const RepoInfo>;
   /** \relates RepoInfo */
-  typedef std::list<RepoInfo> RepoInfoList;
+  using RepoInfoList = std::list<RepoInfo>;
 
   /** \relates RepoInfo Stream output */
-  std::ostream & operator<<( std::ostream & str, const RepoInfo & obj );
+  std::ostream & operator<<( std::ostream & str, const RepoInfo & obj ) ZYPP_API;
 
   /** \relates RepoInfo::GpgCheck Stream output */
-  std::ostream & operator<<( std::ostream & str, const RepoInfo::GpgCheck & obj );
+  std::ostream & operator<<( std::ostream & str, const RepoInfo::GpgCheck & obj ) ZYPP_API;
 
   /////////////////////////////////////////////////////////////////
 } // namespace zypp
