@@ -14,6 +14,7 @@
 #include <iostream>
 #include <typeinfo>
 
+#include <utility>
 #include <zypp-media/MediaException>
 #include <zypp/media/MediaManager.h>
 #include <zypp/media/MediaHandlerFactory.h>
@@ -53,11 +54,12 @@ namespace zypp
           catch(...) {}
         }
 
-        ManagedMedia( ManagedMedia &&m )
-          : desired ( m.desired )
-          , verifier( std::move(m.verifier) )
-          , _handler ( std::move(m._handler) )
-        {}
+        ManagedMedia(const ManagedMedia &) = delete;
+        ManagedMedia &operator=(const ManagedMedia &) = delete;
+
+        ManagedMedia(ManagedMedia &&m) noexcept
+          : desired(m.desired), verifier(std::move(m.verifier)),
+            _handler(std::move(m._handler)) {}
 
         static ManagedMedia makeManagedMedia ( const Url &o_url, const Pathname &preferred_attach_point, const MediaVerifierRef &v )
         {
@@ -161,9 +163,9 @@ namespace zypp
         Pathname         deltafile;
 
       private:
-        ManagedMedia( std::unique_ptr<MediaHandler> &&h, const MediaVerifierRef &v)
+        ManagedMedia( std::unique_ptr<MediaHandler> &&h, MediaVerifierRef v)
           : desired (false)
-          , verifier(v)
+          , verifier(std::move(v))
           , _handler ( std::move(h) )
         {}
 
@@ -175,7 +177,7 @@ namespace zypp
       }
 
       // -------------------------------------------------------------
-      typedef std::map<MediaAccessId, ManagedMedia> ManagedMediaMap;
+      using ManagedMediaMap = std::map<MediaAccessId, ManagedMedia>;
 
       ////////////////////////////////////////////////////////////////
     } // anonymous
@@ -212,13 +214,19 @@ namespace zypp
       {}
 
     public:
+
+      MediaManager_Impl(const MediaManager_Impl &) = delete;
+      MediaManager_Impl(MediaManager_Impl &&) = delete;
+      MediaManager_Impl &operator=(const MediaManager_Impl &) = delete;
+      MediaManager_Impl &operator=(MediaManager_Impl &&) = delete;
+
       ~MediaManager_Impl()
       {
         try
         {
           // remove depending (iso) handlers first
           ManagedMediaMap::iterator it;
-          bool found;
+          bool found = false;
           do
           {
             found = false;
@@ -892,7 +900,7 @@ namespace zypp
         if( ret.mediaSource && ret.attachPoint)
         {
           std::string mnt(ret.attachPoint->path.asString());
-          std::string our(path.asString());
+          const std::string& our(path.asString());
 
           if( our == mnt)
           {
@@ -922,7 +930,7 @@ namespace zypp
       for( e = entries.begin(); e != entries.end(); ++e)
       {
         std::string mnt(Pathname(e->dir).asString());
-        std::string our(path.asString());
+        const std::string& our(path.asString());
 
         if( our == mnt)
         {
