@@ -18,6 +18,7 @@
 #include <arpa/inet.h>
 #include <glib.h>
 
+#include <utility>
 #include <vector>
 #include <iostream>
 #include <algorithm>
@@ -143,7 +144,11 @@ class multifetchworker : private MediaCurl, public zyppng::CurlMultiPartDataRece
 
 public:
   multifetchworker(int no, multifetchrequest &request, const Url &url);
-  ~multifetchworker();
+  multifetchworker(const multifetchworker &) = delete;
+  multifetchworker(multifetchworker &&) = delete;
+  multifetchworker &operator=(const multifetchworker &) = delete;
+  multifetchworker &operator=(multifetchworker &&) = delete;
+  ~multifetchworker() override;
 
   /*!
    * Fetches the next job from the parent request,
@@ -226,7 +231,14 @@ private:
 
 class multifetchrequest : protected internal::CurlPollHelper::CurlPoll  {
 public:
-  multifetchrequest(const MediaMultiCurl *context, const Pathname &filename, const Url &baseurl, CURLM *multi, FILE *fp, callback::SendReport<DownloadProgressReport> *report, MediaBlockList &&blklist, off_t filesize);
+  multifetchrequest(const MediaMultiCurl *context, Pathname filename,
+                    Url baseurl, CURLM *multi, FILE *fp,
+                    callback::SendReport<DownloadProgressReport> *report,
+                    MediaBlockList &&blklist, off_t filesize);
+  multifetchrequest(const multifetchrequest &) = delete;
+  multifetchrequest(multifetchrequest &&) = delete;
+  multifetchrequest &operator=(const multifetchrequest &) = delete;
+  multifetchrequest &operator=(multifetchrequest &&) = delete;
   ~multifetchrequest();
 
   void run(std::vector<Url> &urllist);
@@ -285,6 +297,20 @@ public:
 constexpr auto MIN_REQ_MIRRS = 4;
 constexpr auto MAXURLS       = 10;
 
+// TCP communication scales up as a connection proceeds. This is due to TCP slowstart where
+// the congestion window scales up. The stripe calculation assumes that every package can be fairly
+// downloaded from multiple mirrors omits that attempting to download say 1MB from 4 mirrors
+// means 4 requests of 256k, where then you have four congestion windows that need to increase
+// meaning the overall download speed is significantly lower. Counter intuitively this leads to
+// cases where *more* mirrors being available to zypper significantly lowers performance.
+//
+// Instead, there should be a minimum stripe size cap. This way any item smaller than the value
+// is downloaded in a single request, where as larger items are downloaded from many mirrors
+// but each range has enough time to increase it's congestion window to something reasonable.
+//
+// Initial value 4 MB;
+constexpr auto MIN_STRIPE_SIZE_KB = 4096;
+
 //////////////////////////////////////////////////////////////////////
 
 static double
@@ -406,7 +432,7 @@ multifetchworker::headerfunction( char *p, size_t bytes )
   if ( repSize && *repSize != _request->_filesize  ) {
       XXX << "#" << _workerno << ": filesize mismatch" << endl;
       _state = WORKER_BROKEN;
-      strncpy(_curlError, "filesize mismatch", CURL_ERROR_SIZE);
+      setCurlError("filesize mismatch");
       return 0;
   }
 
@@ -427,7 +453,7 @@ multifetchworker::multifetchworker(int no, multifetchrequest &request, const Url
   if (!_curl && !(_curl = curl_easy_init()))
     {
       _state = WORKER_BROKEN;
-      strncpy(_curlError, "curl_easy_init failed", CURL_ERROR_SIZE);
+      setCurlError("curl_easy_init failed");
       return;
     }
 
@@ -446,7 +472,7 @@ bool multifetchworker::setupHandle()
     curl_easy_cleanup(_curl);
     _curl = 0;
     _state = WORKER_BROKEN;
-    strncpy(_curlError, "curl_easy_setopt failed", CURL_ERROR_SIZE);
+    setCurlError("curl_easy_setopt failed");
     return false;
   }
   curl_easy_setopt(_curl, CURLOPT_PRIVATE, this);
@@ -501,10 +527,8 @@ multifetchworker::~multifetchworker()
   if (_pid)
     {
       kill(_pid, SIGKILL);
-      int status;
-      while (waitpid(_pid, &status, 0) == -1)
-        if (errno != EINTR)
-          break;
+      int status = 0;
+      zyppng::eintrSafeCall( waitpid, _pid, &status, 0);
       _pid = 0;
     }
   if (_dnspipe != -1)
@@ -517,7 +541,7 @@ multifetchworker::~multifetchworker()
   disconnectFrom();
 }
 
-static inline bool env_isset(std::string name)
+static inline bool env_isset(const std::string& name)
 {
   const char *s = getenv(name.c_str());
   return s && *s ? true : false;
@@ -561,7 +585,7 @@ multifetchworker::checkdns()
   if (pipe(pipefds))
     {
       _state = WORKER_BROKEN;
-      strncpy(_curlError, "DNS pipe creation failed", CURL_ERROR_SIZE);
+      setCurlError("DNS pipe creation failed");
       return;
     }
   _pid = fork();
@@ -571,14 +595,14 @@ multifetchworker::checkdns()
       close(pipefds[1]);
       _pid = 0;
       _state = WORKER_BROKEN;
-      strncpy(_curlError, "DNS checker fork failed", CURL_ERROR_SIZE);
+      setCurlError("DNS checker fork failed");
       return;
     }
   else if (_pid == 0)
     {
       close(pipefds[0]);
       // XXX: close all other file descriptors
-      struct addrinfo *ai, aihints;
+      struct addrinfo *ai = nullptr, aihints;
       memset(&aihints, 0, sizeof(aihints));
       aihints.ai_family = PF_UNSPEC;
       int tstsock = socket(PF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
@@ -624,12 +648,8 @@ multifetchworker::dnsevent( const std::vector<GPollFD> &waitFds )
 
   if (_state != WORKER_LOOKUP || !hasEvent)
     return;
-  int status;
-  while (waitpid(_pid, &status, 0) == -1)
-    {
-      if (errno != EINTR)
-        return;
-    }
+  int status = 0;
+  zyppng::eintrSafeCall( waitpid, _pid, &status, 0);
   _pid = 0;
   if (_dnspipe != -1)
     {
@@ -639,7 +659,7 @@ multifetchworker::dnsevent( const std::vector<GPollFD> &waitFds )
   if (!WIFEXITED(status))
     {
       _state = WORKER_BROKEN;
-      strncpy(_curlError, "DNS lookup failed", CURL_ERROR_SIZE);
+      setCurlError("DNS lookup failed");
       _request->_activeworkers--;
       return;
     }
@@ -648,7 +668,7 @@ multifetchworker::dnsevent( const std::vector<GPollFD> &waitFds )
   if (exitcode != 0)
     {
       _state = WORKER_BROKEN;
-      strncpy(_curlError, "DNS lookup failed", CURL_ERROR_SIZE);
+      setCurlError("DNS lookup failed");
       _request->_activeworkers--;
       return;
     }
@@ -901,7 +921,7 @@ bool multifetchworker::continueJob()
 {
   bool hadRangeFail = _multiByteHandler->lastError() == MultiByteHandler::Code::RangeFail;
   if ( !_multiByteHandler->prepareToContinue() ) {
-    strncpy( _curlError, _multiByteHandler->lastErrorMessage().c_str(), CURL_ERROR_SIZE );
+    setCurlError(_multiByteHandler->lastErrorMessage().c_str());
     return false;
   }
 
@@ -930,14 +950,14 @@ multifetchworker::run()
   if ( !_multiByteHandler->prepare() ) {
     _request->_activeworkers--;
     _state = WORKER_BROKEN;
-    strncpy( _curlError, _multiByteHandler->lastErrorMessage ().c_str() , CURL_ERROR_SIZE );
+    setCurlError(_multiByteHandler->lastErrorMessage ().c_str());
     return;
   }
 
   if (curl_multi_add_handle(_request->_multi, _curl) != CURLM_OK) {
     _request->_activeworkers--;
     _state = WORKER_BROKEN;
-    strncpy( _curlError, "curl_multi_add_handle failed", CURL_ERROR_SIZE );
+    setCurlError("curl_multi_add_handle failed");
     return;
   }
 
@@ -949,11 +969,11 @@ multifetchworker::run()
 //////////////////////////////////////////////////////////////////////
 
 
-multifetchrequest::multifetchrequest(const MediaMultiCurl *context, const Pathname &filename, const Url &baseurl, CURLM *multi, FILE *fp, callback::SendReport<DownloadProgressReport> *report, MediaBlockList &&blklist, off_t filesize)
+multifetchrequest::multifetchrequest(const MediaMultiCurl *context, Pathname filename, Url baseurl, CURLM *multi, FILE *fp, callback::SendReport<DownloadProgressReport> *report, MediaBlockList &&blklist, off_t filesize)
   : internal::CurlPollHelper::CurlPoll{ multi }
   , _context(context)
-  , _filename(filename)
-  , _baseurl(baseurl)
+  , _filename(std::move(filename))
+  , _baseurl(std::move(baseurl))
   , _fp(fp)
   , _report(report)
   , _blklist(std::move(blklist))
@@ -1051,7 +1071,7 @@ multifetchrequest::run(std::vector<Url> &urllist)
             {
               if ((*workeriter)->_state != WORKER_BROKEN)
                 continue;
-              ZYPP_THROW(MediaCurlException(_baseurl, "Server error", (*workeriter)->_curlError));
+              ZYPP_THROW(MediaCurlException(_baseurl, "Server error", (*workeriter)->curlError()));
             }
           break;
         }
@@ -1148,15 +1168,15 @@ multifetchrequest::run(std::vector<Url> &urllist)
         }
 
       // collect all curl results, (re)schedule jobs
-      CURLMsg *msg;
-      int nqueue;
+      CURLMsg *msg = nullptr;
+      int nqueue = 0;
       while ((msg = curl_multi_info_read(_multi, &nqueue)) != 0)
         {
           if (msg->msg != CURLMSG_DONE)
             continue;
           CURL *easy = msg->easy_handle;
           CURLcode cc = msg->data.result;
-          multifetchworker *worker;
+          multifetchworker *worker = nullptr;
 
           if (curl_easy_getinfo(easy, CURLINFO_PRIVATE, &worker) != CURLE_OK)
             ZYPP_THROW(MediaCurlException(_baseurl, "curl_easy_getinfo", "unknown error"));
@@ -1174,7 +1194,7 @@ multifetchrequest::run(std::vector<Url> &urllist)
           const auto &setWorkerBroken = [&]( const std::string &str = {} ){
             worker->_state = WORKER_BROKEN;
             if ( !str.empty () )
-              strncpy(worker->_curlError, str.c_str(), CURL_ERROR_SIZE);
+              worker->setCurlError(str.c_str());
             _activeworkers--;
 
             if (!_activeworkers && !(urliter != urllist.end() && _workers.size() < MAXURLS)) {
@@ -1333,7 +1353,9 @@ multifetchrequest::run(std::vector<Url> &urllist)
 
 inline zypp::ByteCount multifetchrequest::makeBlksize ( uint maxConns, size_t filesize )
 {
-  return std::max<zypp::ByteCount>( filesize / std::min( std::max<int>( 1, maxConns ) , MAXURLS ), zypp::ByteCount(4, zypp::ByteCount::K) );
+  // If the calculated strip size is too small and can cause a loss in TCP throughput. Raise
+  // it to a reasonable value.
+  return std::max<zypp::ByteCount>( filesize / std::min( std::max<int>( 1, maxConns ) , MAXURLS ), zypp::ByteCount(MIN_STRIPE_SIZE_KB, zypp::ByteCount::K) );
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -1616,7 +1638,7 @@ void MediaMultiCurl::doGetFileCopy( const OnMediaLocation &srcFile , const Pathn
               XXX << bl << endl;
               filesystem::unlink(failedFile);
             }
-          Pathname df = srcFile.deltafile();
+          const Pathname& df = srcFile.deltafile();
           if (!df.empty())
             {
               XXX << "reusing blocks from file " << df << endl;
@@ -1758,7 +1780,7 @@ void MediaMultiCurl::checkFileDigest(Url &url, FILE *fp, MediaBlockList &blklist
   Digest dig;
   blklist.createFileDigest(dig);
   char buf[4096];
-  size_t l;
+  size_t l = 0;
   while ((l = fread(buf, 1, sizeof(buf), fp)) > 0)
     dig.update(buf, l);
   if (!blklist.verifyFileDigest(dig))
