@@ -24,7 +24,8 @@
 #include <zypp/sat/LookupAttr.h>
 
 #include <zypp/Repository.h>
-#include <zypp/OnMediaLocation.h>
+#include <utility>
+#include <zypp-core/OnMediaLocation>
 #include <zypp/ZConfig.h>
 
 #include <zypp/ui/Selectable.h>
@@ -45,26 +46,26 @@ namespace zypp
         if ( ! _ident )
           return;
 
-	ResKind explicitKind = ResKind::explicitBuiltin( _ident.c_str() );
-	// NOTE: kind package and srcpackage do not have namespaced ident!
-	if ( ! explicitKind  )
-	{
+        ResKind explicitKind = ResKind::explicitBuiltin( _ident.c_str() );
+        // NOTE: kind package and srcpackage do not have namespaced ident!
+        if ( ! explicitKind  )
+        {
           _name = _ident;
-	  // No kind defaults to package
-	  if ( !_kind )
-	    _kind = ResKind::package;
-	  else if ( ! ( _kind == ResKind::package || _kind == ResKind::srcpackage ) )
-	    _ident = IdString( str::form( "%s:%s", _kind.c_str(), _ident.c_str() ) );
-	}
-	else
-	{
-	  // strip kind spec from name
-	  _name = IdString( ::strchr( _ident.c_str(), ':' )+1 );
-	  _kind = explicitKind;
-	  if ( _kind == ResKind::package || _kind == ResKind::srcpackage )
-	    _ident = _name;
-	}
-	return;
+          // No kind defaults to package
+          if ( !_kind )
+            _kind = ResKind::package;
+          else if ( ! ( _kind == ResKind::package || _kind == ResKind::srcpackage ) )
+            _ident = IdString( str::form( "%s:%s", _kind.c_str(), _ident.c_str() ) );
+        }
+        else
+        {
+          // strip kind spec from name
+          _name = IdString( ::strchr( _ident.c_str(), ':' )+1 );
+          _kind = explicitKind;
+          if ( _kind == ResKind::package || _kind == ResKind::srcpackage )
+            _ident = _name;
+        }
+        return;
       }
     } // namespace
     ///////////////////////////////////////////////////////////////////
@@ -83,12 +84,12 @@ namespace zypp
 
     Solvable::SplitIdent::SplitIdent( ResKind kind_r, IdString name_r )
     : _ident( name_r )
-    , _kind( kind_r )
+    , _kind(std::move( kind_r ))
     { _doSplit( _ident, _kind, _name ); }
 
     Solvable::SplitIdent::SplitIdent( ResKind kind_r, const C_Str & name_r )
     : _ident( name_r )
-    , _kind( kind_r )
+    , _kind(std::move( kind_r ))
     { _doSplit( _ident, _kind, _name ); }
 
     /////////////////////////////////////////////////////////////////
@@ -98,7 +99,8 @@ namespace zypp
     const Solvable Solvable::noSolvable;
 
     const IdString Solvable::retractedToken	{ "retracted-patch-package()" };
-    const IdString Solvable::ptfToken		{ "ptf()" };
+    const IdString Solvable::ptfMasterToken	{ "ptf()" };
+    const IdString Solvable::ptfPackageToken	{ "ptf-package()" };
 
     /////////////////////////////////////////////////////////////////
 
@@ -143,13 +145,13 @@ namespace zypp
       }
       else
       {
-	for ( Locale l( lang_r ); l; l = l.fallback() )
-	{
-	  if ( (s = ::solvable_lookup_str_lang( _solvable, attr.id(), l.c_str(), 0 )) )
-	    return s;
-	}
-	// here: no matching locale, so use default
-	s = ::solvable_lookup_str_lang( _solvable, attr.id(), 0, 0 );
+        for ( Locale l( lang_r ); l; l = l.fallback() )
+        {
+          if ( (s = ::solvable_lookup_str_lang( _solvable, attr.id(), l.c_str(), 0 )) )
+            return s;
+        }
+        // here: no matching locale, so use default
+        s = ::solvable_lookup_str_lang( _solvable, attr.id(), 0, 0 );
       }
       return s ? s : std::string();
    }
@@ -224,12 +226,12 @@ namespace zypp
     {
       NO_SOLVABLE_RETURN( OnMediaLocation() );
       // medianumber and path
-      unsigned medianr;
+      unsigned medianr = 0;
       const char * file = ::solvable_lookup_location( _solvable, &medianr );
       if ( ! file )
         return OnMediaLocation();
       if ( ! medianr )
-	medianr = 1;
+        medianr = 1;
 
       OnMediaLocation ret;
 
@@ -287,12 +289,12 @@ namespace zypp
       const char * ident = IdString( _solvable->name ).c_str();
       ResKind knownKind( ResKind::explicitBuiltin( ident ) );
       if ( knownKind )
-	return knownKind;
+        return knownKind;
 
       // ...or no ':' in package names (hopefully)...
       const char * sep = ::strchr( ident, ':' );
       if ( ! sep )
-	return ResKind::package;
+        return ResKind::package;
 
       // ...or something unknown.
       return ResKind( std::string( ident, sep-ident ) );
@@ -396,22 +398,36 @@ namespace zypp
       return myPool().isNeedreboot( *this );
     }
 
+    // TODO: Optimize
+    bool Solvable::isBlacklisted() const
+    { return isPtf() || isRetracted(); }
+
     bool Solvable::isRetracted() const
     {
       NO_SOLVABLE_RETURN( false );
       if ( isKind<Package>() )
-	return provides().contains( Capability( retractedToken.id() ) );
+        return myPool().isRetracted( *this );
       if ( isKind<Patch>() )
-	return lookupStrAttribute( SolvAttr::updateStatus ) == "retracted";
+        return lookupStrAttribute( SolvAttr::updateStatus ) == "retracted";
       return false;
     }
 
     bool Solvable::isPtf() const
+    { return isPtfPackage() || isPtfMaster(); }
+
+    bool Solvable::isPtfMaster() const
     {
       NO_SOLVABLE_RETURN( false );
-      return provides().contains( Capability( ptfToken.id() ) );
+      return myPool().isPtfMaster( *this );
     }
 
+    bool Solvable::isPtfPackage() const
+    {
+      NO_SOLVABLE_RETURN( false );
+      return myPool().isPtfPackage( *this );
+    }
+
+
     bool Solvable::multiversionInstall() const
     {
       NO_SOLVABLE_RETURN( false );
@@ -461,7 +477,7 @@ namespace zypp
     {
       inline Capabilities _getCapabilities( detail::IdType * idarraydata_r, ::Offset offs_r )
       {
-	return offs_r ? Capabilities( idarraydata_r + offs_r ) : Capabilities();
+        return offs_r ? Capabilities( idarraydata_r + offs_r ) : Capabilities();
       }
     } // namespace
     ///////////////////////////////////////////////////////////////////
@@ -566,7 +582,7 @@ namespace zypp
        * dependency. Return #invocations of fnc_r, negative if fnc_r returned
        * false to indicate abort.
        */
-      int invokeOnEachSupportedLocale( Capability cap_r, function<bool (const Locale &)> fnc_r )
+      int invokeOnEachSupportedLocale( Capability cap_r, const function<bool (const Locale &)>& fnc_r )
       {
         CapDetail detail( cap_r );
         if ( detail.kind() == CapDetail::EXPRESSION )
@@ -594,9 +610,7 @@ namespace zypp
               }
               break;
 
-            case CapDetail::REL_NONE:
-            case CapDetail::CAP_WITH:
-            case CapDetail::CAP_ARCH:
+            default:
               break; // unwanted
           }
         }
@@ -607,7 +621,7 @@ namespace zypp
        * dependency. Return #invocations of fnc_r, negative if fnc_r returned
        * false to indicate abort.
        */
-      inline int invokeOnEachSupportedLocale( Capabilities cap_r, function<bool (Locale)> fnc_r )
+      inline int invokeOnEachSupportedLocale( Capabilities cap_r, const function<bool (Locale)>& fnc_r )
       {
         int cnt = 0;
         for_( cit, cap_r.begin(), cap_r.end() )
@@ -681,7 +695,7 @@ namespace zypp
       if ( ! file )
         medianr = 0U;
       else if ( ! medianr )
-	medianr = 1U;
+        medianr = 1U;
       return medianr;
     }
 
@@ -733,13 +747,13 @@ namespace zypp
       std::string ret = lookupStrAttribute( SolvAttr::eula, lang_r );
       if ( ret.empty() && isKind<Product>() )
       {
-	const RepoInfo & ri( repoInfo() );
-	std::string riname( name() );	// "license-"+name with fallback "license"
-	if ( ! ri.hasLicense( riname ) )
-	  riname.clear();
+        const RepoInfo & ri( repoInfo() );
+        std::string riname( name() );	// "license-"+name with fallback "license"
+        if ( ! ri.hasLicense( riname ) )
+          riname.clear();
 
-	if ( ri.needToAcceptLicense( riname ) || ! ui::Selectable::get( *this )->hasInstalledObj() )
-	  ret = ri.getLicense( riname, lang_r ); // bnc#908976: suppress informal license upon update
+        if ( ri.needToAcceptLicense( riname ) || ! ui::Selectable::get( *this )->hasInstalledObj() )
+          ret = ri.getLicense( riname, lang_r ); // bnc#908976: suppress informal license upon update
       }
       return ret;
     }
@@ -749,12 +763,12 @@ namespace zypp
       NO_SOLVABLE_RETURN( false );
       if ( isKind<Product>() )
       {
-	const RepoInfo & ri( repoInfo() );
-	std::string riname( name() );	// "license-"+name with fallback "license"
-	if ( ! ri.hasLicense( riname ) )
-	  riname.clear();
+        const RepoInfo & ri( repoInfo() );
+        std::string riname( name() );	// "license-"+name with fallback "license"
+        if ( ! ri.hasLicense( riname ) )
+          riname.clear();
 
-	return ri.needToAcceptLicense( riname );
+        return ri.needToAcceptLicense( riname );
       }
       return true;
     }
